#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <string.h>
#include <sys/time.h>

#include "print_const.h"

#define LOG_FILE "snmp_time.log"
#define TIME_BUFF_SIZE 32

netsnmp_session session, *ss;
netsnmp_pdu *pdu;
netsnmp_pdu *response;
size_t anOID_len;
size_t currOID_len;
int endloop = 0;
int loop = 0;
FILE *logfile;

struct tm *tm_before, *tm_after, *tm_before_row, *tm_after_row;
time_t time_before, time_after, time_before_row, time_after_row; 

int log_time(struct tm *tm_before, struct tm *tm_after, oid *oid, char *before, char *after){
	if(!logfile) return;
	char time_buff_before[TIME_BUFF_SIZE], time_buff_after[TIME_BUFF_SIZE];
	
	strftime (time_buff_before, sizeof(time_buff_before), "%Y-%m-%d %H:%M:%S",  tm_before);
	strftime (time_buff_after, sizeof(time_buff_after), "%Y-%m-%d %H:%M:%S",  tm_after);
    
	fprintf(logfile, "%sRequest at %s, response at %s %s", before, time_buff_before, time_buff_after, after);
    if(oid){
		int i;
		fprintf(logfile," for OID = ");
		for (i = 0; i < currOID_len-1; i++){
		    fprintf(logfile, "%d.", (int) *(oid + i));
		}
		fprintf(logfile, "%d", (int) *(oid + i));
	}
	fprintf(logfile,"\n");
}

void single_request(oid *item, int type){
	if(endloop) return;

	netsnmp_variable_list *vars;
    int status;
	pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
    snmp_add_null_var(pdu, item, anOID_len);

	time(&time_before);
    status = snmp_synch_response(ss, pdu, &response);
	//sleep(1);
	time(&time_after);time(&time_after_row);

	tm_before = localtime(&time_before);
    tm_after = localtime(&time_after);

    /* Process the response. */
    if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {/* SUCCESS */

		for(vars = response->variables; vars; vars = vars->next_variable){
			currOID_len = vars->name_length;
			int k;
			for(k = 0; k<vars->name_length-1; k++){
				if((int)item[k]!=(int)vars->name[k]){
					endloop = 1;
					return;
				}
			}
			/*   
			print_variable(vars->name, vars->name_length, vars);
			*/
			int i;
			switch (type){
				case 1: //hex (physAddr)
					printf(" %02x:%02x:%02x:%02x | ",*(vars->val.string), *(vars->val.string+1), *(vars->val.string+2), *(vars->val.string+3));
					break;
				case 2:
					for (i=0;i<vars->val_len;i++){
						printf("%c",(char)*(vars->val.string+i));
				   	}
					break;
				case 3: //integer
					printf("\t  %2ld   | ",*(vars->val.integer));
					break;
				case 4: //integer
					if(*(vars->val.integer))
						printf("   UP   | ");
					else
						printf("  DOWN  | ");
					   	break;
			}
			log_time(tm_before, tm_after, item, NOTHING_MSG, NOTHING_MSG);

			fprintf(logfile,"\tResult: ");
			fprint_variable(logfile, vars->name, vars->name_length, vars);
		}
    } else {
      if (status == STAT_SUCCESS)
        fprintf(stderr, "Error in packet\nReason: %s\n", snmp_errstring(response->errstat));
      else if (status == STAT_TIMEOUT)
        fprintf(stderr, "Timeout: No response from %s.\n", session.peername);
      else
        snmp_sess_perror("snmpdemoapp", ss);
	  endloop = 1;
	  return;
    }
    if (response)
      snmp_free_pdu(response);

	
}

int main(int argc, char ** argv)
{
	if(argc < 3) { print_program_usage(); return 0; }

	oid tab_index[MAX_OID_LEN] = 		{1,3,6,1,2,1,2,2,1,1};
	oid tab_physAddr[MAX_OID_LEN] = 	{1,3,6,1,2,1,2,2,1,6};
	oid tab_descr[MAX_OID_LEN] = 		{1,3,6,1,2,1,2,2,1,2};
	oid tab_admStatus[MAX_OID_LEN] = 	{1,3,6,1,2,1,2,2,1,7};
	anOID_len = MAX_OID_LEN;

	char *param_host = argv[1];
	char *param_community = argv[2];

	logfile = fopen(LOG_FILE, "a");
	if(!logfile)
        printf("Cannot open log file!\n");
	else
		fprintf(logfile,"\n%s\nPROGRAM STARTED\tSNMP IF_MIB::ifTable\n%s\n",ROW_DELIM_LONG,ROW_DELIM_LONG);

    init_snmp("snmpneumann");
    snmp_sess_init( &session );                   /* set up defaults */
    session.peername = strdup(param_host);
    session.version = SNMP_VERSION_2c;
    session.community = param_community;
    session.community_len = strlen(session.community);
    ss = snmp_open(&session);                     /* establish the session */
    if (!ss) {
      snmp_sess_perror("ack", &session);
      SOCK_CLEANUP;
      exit(1);
    }

	print_table_header();
	while(!endloop){
		tab_physAddr[10] = tab_descr[10] = tab_index[10] =  loop;
		time(&time_before_row);
		single_request(tab_index, 3);	
		single_request(tab_admStatus, 4);	
		single_request(tab_physAddr, 1);
		single_request(tab_descr, 2);
		
		loop++;

		if(!endloop) {
			printf("\n");

			tm_before_row = localtime(&time_before_row);
			tm_after_row = localtime(&time_after_row);

			log_time(tm_before_row, tm_after_row, NULL, "ROW: ", ROW_DELIM_SLIM_LONG);
		}
	}printf("%s\n", ROW_DELIM);

	if(logfile)
        fclose(logfile);

    snmp_close(ss);
    return 0;
}

